@@ -37,7 +37,7 @@ typedef void (*TransformCallback) (PangoContext *context,
 				   PangoMatrix  *transform,
 				   gpointer      data);
 
-void fail (const char *format, ...) G_GNUC_PRINTF (1, 2);
+void fail (const char *format, ...) G_GNUC_PRINTF (1, 2) G_GNUC_NORETURN;
 
 void   parse_options      (int               argc,
 			   char             *argv[]);
@@ -47,6 +47,8 @@ void   do_output          (PangoContext     *context,
 			   gpointer          cb_data,
 			   int              *width,
 			   int              *height);
+void   update             (void);
+void   finalize           (void);
 void   fc_substitute_func (FcPattern        *pattern,
 			   gpointer          data);
 gchar *get_options_string (void);
@@ -55,7 +57,7 @@ extern char *prog_name;
 
 extern gboolean opt_display;
 extern int opt_dpi;
-extern char *opt_font;
+extern const char *opt_font;
 extern gboolean opt_header;
 extern char *opt_output;
 extern int opt_margin;
@@ -67,5 +69,6 @@ extern char *opt_text;
 extern gboolean opt_waterfall;
 extern int opt_width;
 extern int opt_indent;
+extern int opt_runs;
 extern PangoEllipsizeMode opt_ellipsize;
 extern HintMode opt_hinting;
