@@ -64,8 +64,8 @@ void indic_mprefixups_apply(MPreFixups *mprefixups, PangoOTBuffer *buffer)
     glong fixup;
 
     for (fixup = 0; fixup < mprefixups->fFixupCount; fixup += 1) {
-	glong baseIndex = mprefixups->fFixupData[fixup].fBaseIndex;
-	glong mpreIndex = mprefixups->fFixupData[fixup].fMPreIndex;
+	gulong baseIndex = mprefixups->fFixupData[fixup].fBaseIndex;
+	gulong mpreIndex = mprefixups->fFixupData[fixup].fMPreIndex;
 	glong baseGlyph = -1;
 	glong mpreGlyph = -1;
 	glong mpreLimit = -1;
@@ -74,6 +74,7 @@ void indic_mprefixups_apply(MPreFixups *mprefixups, PangoOTBuffer *buffer)
 	PangoOTGlyph *glyphs;
 	int n_glyphs;
 	PangoOTGlyph *mpreSave;
+	int *clusterSave;
 
 	/* determine post GSUB location of baseIndex and mpreIndex */
 
@@ -97,6 +98,7 @@ void indic_mprefixups_apply(MPreFixups *mprefixups, PangoOTBuffer *buffer)
 	mpreDest   = baseGlyph - mpreCount;
 
 	mpreSave    = g_new (PangoOTGlyph, mpreCount);
+	clusterSave = g_new (int, mpreCount);
 
 	for (i = 0; i < mpreCount; i += 1) {
 	    mpreSave[i] = glyphs[mpreGlyph + i];
