@@ -55,6 +55,7 @@ pango_ot_ruleset_get_type (void)
         sizeof (PangoOTRuleset),
         0,              /* n_preallocs */
         (GInstanceInitFunc)pango_ot_ruleset_init,
+	NULL            /* value_table */
       };
       
       object_type = g_type_register_static (G_TYPE_OBJECT,
@@ -93,11 +94,9 @@ pango_ot_ruleset_finalize (GObject *object)
 /**
  * pango_ot_ruleset_new:
  * @info: a #PangoOTInfo.
+ * @returns: a new #PangoOTRuleset.
  *
  * Creates a new #PangoOTRuleset for the given OpenType info.
- *
- * Return value: the newly allocated #PangoOTRuleset, which
- *               should be freed with g_object_unref().
  **/
 PangoOTRuleset *
 pango_ot_ruleset_new (PangoOTInfo *info)
@@ -138,117 +137,72 @@ pango_ot_ruleset_add_feature (PangoOTRuleset   *ruleset,
   g_array_append_val (ruleset->rules, tmp_rule);
 }
 
-/**
- * pango_ot_ruleset_substitute:
- * @ruleset: a #PangoOTRuleset.
- * @buffer: a #PangoOTBuffer.
- *
- * Performs the OpenType GSUB substitution on @buffer using the features
- * in @ruleset
- *
- * Since: 1.4
- **/
 void
 pango_ot_ruleset_substitute  (PangoOTRuleset   *ruleset,
 			      PangoOTBuffer    *buffer)
 {
-  int i;
+  unsigned int i;
   
   TTO_GSUB gsub = NULL;
   
-  gboolean need_gsub = FALSE;
-
   g_return_if_fail (PANGO_OT_IS_RULESET (ruleset));
 
   for (i = 0; i < ruleset->rules->len; i++)
     {
       PangoOTRule *rule = &g_array_index (ruleset->rules, PangoOTRule, i);
 
-      if (rule->table_type == PANGO_OT_TABLE_GSUB)
-	need_gsub = TRUE;
-    }
+      if (rule->table_type != PANGO_OT_TABLE_GSUB)
+	continue;
 
-  if (need_gsub)
-    {
-
-      gsub = pango_ot_info_get_gsub (ruleset->info);
-
-      if (gsub)
-	TT_GSUB_Clear_Features (gsub);
-    }
-
-  for (i = 0; i < ruleset->rules->len; i++)
-    {
-      PangoOTRule *rule = &g_array_index (ruleset->rules, PangoOTRule, i);
-
-      if (rule->table_type == PANGO_OT_TABLE_GSUB)
+      if (!gsub)
 	{
-	  if (gsub)
-	    TT_GSUB_Add_Feature (gsub, rule->feature_index, rule->property_bit);
-	}
-    }
+	  gsub = pango_ot_info_get_gsub (ruleset->info);
 
-  if (!gsub)
-    return;
+	  if (gsub)
+	    TT_GSUB_Clear_Features (gsub);
+	  else
+	    return;
+	}
+
+      TT_GSUB_Add_Feature (gsub, rule->feature_index, rule->property_bit);
+    }
 
   TT_GSUB_Apply_String (gsub, buffer->buffer);
 }
 
-/**
- * pango_ot_ruleset_position:
- * @ruleset: a #PangoOTRuleset.
- * @buffer: a #PangoOTBuffer.
- *
- * Performs the OpenType GPOS positionoing on @buffer using the features
- * in @ruleset
- *
- * Since: 1.4
- **/
 void
 pango_ot_ruleset_position (PangoOTRuleset   *ruleset,
 			   PangoOTBuffer    *buffer)
 {
-  int i;
+  unsigned int i;
   
   TTO_GPOS gpos = NULL;
   
-  gboolean need_gpos = FALSE;
-
   g_return_if_fail (PANGO_OT_IS_RULESET (ruleset));
 
   for (i = 0; i < ruleset->rules->len; i++)
     {
       PangoOTRule *rule = &g_array_index (ruleset->rules, PangoOTRule, i);
 
-      if (rule->table_type == PANGO_OT_TABLE_GPOS)
-	need_gpos = TRUE;
+      if (rule->table_type != PANGO_OT_TABLE_GPOS)
+	continue;
+
+      if (!gpos)
+        {
+	  gpos = pango_ot_info_get_gpos (ruleset->info);
+
+	  if (gpos)
+	    TT_GPOS_Clear_Features (gpos);
+	  else
+	    return;
+        }
+
+      TT_GPOS_Add_Feature (gpos, rule->feature_index, rule->property_bit);
     }
 
-  if (need_gpos)
-    gpos = pango_ot_info_get_gpos (ruleset->info);
-
-  if (gpos)
-    {
-      TT_GPOS_Clear_Features (gpos);
-
-      for (i = 0; i < ruleset->rules->len; i++)
-	{
-	  PangoOTRule *rule = &g_array_index (ruleset->rules, PangoOTRule, i);
-	  
-	  if (rule->table_type == PANGO_OT_TABLE_GPOS)
-	    TT_GPOS_Add_Feature (gpos, rule->feature_index, rule->property_bit);
-	}
-    }
-
-  /* Apply GPOS rules */
-  if (gpos)
-    {
-      if (TT_GPOS_Apply_String (ruleset->info->face, gpos, 0, buffer->buffer,
-				FALSE /* enable device-dependant values */,
-				buffer->rtl) == FT_Err_Ok)
-	{
-	  buffer->applied_gpos = TRUE;
-	}
-    }
+  if (TT_GPOS_Apply_String (ruleset->info->face, gpos, 0, buffer->buffer,
+			    FALSE /* enable device-dependant values */,
+			    buffer->rtl) == FT_Err_Ok)
+    buffer->applied_gpos = TRUE;
 }
 
