@@ -43,10 +43,10 @@ PANGO_DEFINE_TYPE_ABSTRACT (PangoFontset, pango_fontset,
 /**
  * pango_fontset_get_font:
  * @fontset: a #PangoFontset
- * @wc: a unicode character
+ * @wc: a Unicode character
  *
  * Returns the font in the fontset that contains the best glyph for the
- * unicode character wc.
+ * Unicode character @wc.
  *
  * Returns: a #PangoFont. The caller must call g_object_unref when finished
  *          with the font.
@@ -67,7 +67,7 @@ pango_fontset_get_font (PangoFontset  *fontset,
  * 
  * Get overall metric information for the fonts in the fontset.
  *
- * Returns: a #PangoMetrics object. The caller must call pango_font_metrics_unref()
+ * Returns: a #PangoFontMetrics object. The caller must call pango_font_metrics_unref()
  *   when finished using the object.
  **/
 PangoFontMetrics *
@@ -100,6 +100,29 @@ pango_fontset_foreach (PangoFontset           *fontset,
   PANGO_FONTSET_GET_CLASS (fontset)->foreach (fontset, func, data);
 }
 
+static gboolean
+get_first_metrics_foreach (PangoFontset  *fontset,
+			   PangoFont     *font,
+			   gpointer       data)
+{
+  PangoFontMetrics *fontset_metrics = data;
+  PangoLanguage *language = PANGO_FONTSET_GET_CLASS (fontset)->get_language (fontset);
+  PangoFontMetrics *font_metrics = pango_font_get_metrics (font, language);
+  guint save_ref_count;
+
+  /* Initialize the fontset metrics to metrics of the first font in the
+   * fontset; saving the refcount and restoring it is a bit of hack but avoids
+   * having to update this code for each metrics addition.
+   */
+  save_ref_count = fontset_metrics->ref_count;
+  *fontset_metrics = *font_metrics;
+  fontset_metrics->ref_count = save_ref_count;
+
+  pango_font_metrics_unref (font_metrics);
+
+  return TRUE;			/* Stops iteration */
+}
+
 static PangoFontMetrics *
 pango_fontset_real_get_metrics (PangoFontset  *fontset)
 {
@@ -118,6 +141,9 @@ pango_fontset_real_get_metrics (PangoFontset  *fontset)
   metrics = pango_font_metrics_new ();
   fonts_seen = g_hash_table_new_full (NULL, NULL, g_object_unref, NULL);
 
+  /* Initialize the metrics from the first font in the fontset */
+  pango_fontset_foreach (fontset, get_first_metrics_foreach, metrics);
+
   p = sample_str;
   while (*p)
     {
@@ -248,7 +274,7 @@ pango_fontset_simple_finalize (GObject *object)
 {
   PangoFontsetSimple *fontset = PANGO_FONTSET_SIMPLE (object);
   PangoCoverage *coverage;
-  int i;
+  unsigned int i;
 
   for (i = 0; i < fontset->fonts->len; i++)
     g_object_unref (g_ptr_array_index(fontset->fonts, i));
@@ -326,7 +352,7 @@ pango_fontset_simple_get_font (PangoFontset  *fontset,
   PangoFont *font;
   PangoCoverage *coverage;
   int result = -1;
-  int i;
+  unsigned int i;
   
   for (i = 0; i < simple->fonts->len; i++)
     {
@@ -361,7 +387,7 @@ pango_fontset_simple_foreach (PangoFontset           *fontset,
 			      gpointer                data)
 {
   PangoFontsetSimple *simple = PANGO_FONTSET_SIMPLE (fontset);
-  int i;
+  unsigned int i;
   
   for (i = 0; i < simple->fonts->len; i++)
     {
