@@ -82,6 +82,32 @@ struct _PangoScriptIter
   int paren_sp;
 };
 
+#define PANGO_SCRIPT_TABLE_MIDPOINT (G_N_ELEMENTS (pango_script_table) / 2)
+
+static inline PangoScript
+pango_script_for_unichar_bsearch (gunichar ch)
+{
+  int lower = 0;
+  int upper = G_N_ELEMENTS (pango_script_table) - 1;
+  static int mid = PANGO_SCRIPT_TABLE_MIDPOINT;
+
+
+  do 
+    {
+      if (ch < pango_script_table[mid].start)
+	upper = mid - 1;
+      else if (ch >= pango_script_table[mid].start + pango_script_table[mid].chars)
+	lower = mid + 1;
+      else
+	return pango_script_table[mid].script;
+
+      mid = (lower + upper) / 2;
+    }
+  while (lower <= upper);
+
+  return PANGO_SCRIPT_COMMON;
+}
+
 /**
  * pango_script_for_unichar:
  * @ch: a Unicode character
@@ -92,27 +118,14 @@ struct _PangoScriptIter
  * result is undefined.
  * 
  * Return value: the #PangoScript for the character.
- *
- * Since: 1.4
  **/
 PangoScript
 pango_script_for_unichar (gunichar ch)
 {
-  int lower = 0;
-  int upper = G_N_ELEMENTS (pango_script_table) - 1;
-
-  while (lower <= upper)
-    {
-      int mid = (lower + upper) / 2;
-      if (ch < pango_script_table[mid].start)
-	upper = mid - 1;
-      else if (ch >= pango_script_table[mid].start + pango_script_table[mid].chars)
-	lower = mid + 1;
-      else
-	return pango_script_table[mid].script;
-    }
-  
-  return PANGO_SCRIPT_COMMON;
+  if (ch < PANGO_EASY_SCRIPTS_RANGE)
+    return pango_script_easy_table[ch];
+  else 
+    return pango_script_for_unichar_bsearch (ch); 
 }
 
 /**********************************************************************/
@@ -130,8 +143,6 @@ pango_script_for_unichar (gunichar ch)
  * Return value: the new script iterator, initialized
  *  to point at the first range in the text. If the string is
  *  empty, it will point at an empty range.
- *
- * Since: 1.4
  **/
 PangoScriptIter *
 pango_script_iter_new (const char *text,
@@ -161,8 +172,6 @@ pango_script_iter_new (const char *text,
  * @iter: a #PangoScriptIter
  * 
  * Frees a #PangoScriptIter created with pango_script_iter_new().
- *
- * Since: 1.4
  **/
 void
 pango_script_iter_free (PangoScriptIter *iter)
@@ -180,8 +189,6 @@ pango_script_iter_free (PangoScriptIter *iter)
  * Gets information about the range to which @iter currently points.
  * The range is the set of locations p where *start <= p < *end.
  * (That is, it doesn't include the character stored at *end)
- *
- * Since: 1.4
  **/
 void
 pango_script_iter_get_range (PangoScriptIter      *iter,
@@ -255,8 +262,6 @@ get_pair_index (gunichar ch)
  * is returned.
  * 
  * Return value: %TRUE if the iter was succesfully advanced.
- *
- * Since: 1.4
  **/
 gboolean
 pango_script_iter_next (PangoScriptIter *iter)
@@ -269,15 +274,18 @@ pango_script_iter_next (PangoScriptIter *iter)
   start_sp = iter->paren_sp;
   iter->script_code = PANGO_SCRIPT_COMMON;
   iter->script_start = iter->script_end;
-  
+
   for (; iter->script_end < iter->text_end; iter->script_end = g_utf8_next_char (iter->script_end))
     {
       gunichar ch = g_utf8_get_char (iter->script_end);
       PangoScript sc;
       int pair_index;
-
+      
       sc = pango_script_for_unichar (ch);
-      pair_index = get_pair_index (ch);
+      if (sc != PANGO_SCRIPT_COMMON)
+	pair_index = -1;
+      else
+	pair_index = get_pair_index (ch);
 
       /*
        * Paired character handling:
@@ -421,7 +429,7 @@ pango_language_includes_script (PangoLanguage *language,
 				PangoScript    script)
 {
   PangoScriptForLang *script_for_lang;
-  int j;
+  unsigned int j;
 
   g_return_val_if_fail (language != NULL, FALSE);
 
@@ -561,7 +569,7 @@ pango_script_get_sample_language (PangoScript script)
   const char *sample_language;
   
   g_return_val_if_fail (script >= 0, NULL);
-  g_return_val_if_fail (script < G_N_ELEMENTS (sample_languages), NULL);
+  g_return_val_if_fail ((guint)script < G_N_ELEMENTS (sample_languages), NULL);
   
   sample_language = sample_languages[script];
 
