@@ -32,7 +32,8 @@ typedef struct _PangoLogAttr PangoLogAttr;
 typedef struct _PangoEngineLang PangoEngineLang;
 typedef struct _PangoEngineShape PangoEngineShape;
 
-typedef struct _PangoFont PangoFont;
+typedef struct _PangoFont    PangoFont;
+typedef struct _PangoFontMap PangoFontMap;
 
 typedef struct _PangoMatrix    PangoMatrix;
 typedef struct _PangoRectangle PangoRectangle;
@@ -146,7 +147,7 @@ void pango_matrix_concat    (PangoMatrix       *matrix,
  * @PANGO_DIRECTION_TTB_LTR: Deprecated value; treated the
  *   same as %PANGO_DIRECTION_RTL.
  * @PANGO_DIRECTION_TTB_RTL: Deprecated value; treated the
- *   same as PANGO_DIRECTION_LTR
+ *   same as %PANGO_DIRECTION_LTR
  * @PANGO_DIRECTION_WEAK_LTR: A weak left-to-right direction
  * @PANGO_DIRECTION_WEAK_RTL: A weak right-to-left direction
  * @PANGO_DIRECTION_NEUTRAL: No direction specified
@@ -159,7 +160,7 @@ void pango_matrix_concat    (PangoMatrix       *matrix,
  * or %PANGO_DIRECTION_WEAK_RTL, since every character is either
  * neutral or has a strong direction; on the other hand
  * %PANGO_DIRECTION_NEUTRAL doesn't make sense to pass
- * to pango_log2vis_get_embedding_levels().
+ * to pango_itemize_with_base_dir().
  *
  * The %PANGO_DIRECTION_TTB_LTR, %PANGO_DIRECTION_TTB_RTL
  * values come from an earlier interpretation of this
