@@ -31,7 +31,7 @@
 
 #include <glib/gstdio.h>
 
-#include "mini-fribidi/fribidi_types.h"
+#include <mini-fribidi/fribidi.h>
 
 #ifndef HAVE_FLOCKFILE
 #  define flockfile(f) (void)1
@@ -421,7 +421,7 @@ pango_scan_string (const char **pos, GString *out)
 gboolean
 pango_scan_int (const char **pos, int *out)
 {
-  int i = 0;
+  unsigned int i = 0;
   char buf[32];
   const char *p = *pos;
 
@@ -1013,8 +1013,8 @@ pango_matrix_get_type (void)
  * 
  * Copies a #PangoMatrix.
  * 
- * Return value: the newly allocated #PangoMatrix, which should
- *               be freed with pango_matrix_free().
+ * Return value: a copy of @matrix. The result must be freed with
+ *  pango_matrix_free().
  *
  * Since: 1.6
  **/
@@ -1154,61 +1154,6 @@ pango_matrix_concat (PangoMatrix       *matrix,
   matrix->y0  = tmp.yx * new_matrix->y0 + tmp.yy * new_matrix->y0 + tmp.y0;
 }
 
-/**
- * pango_matrix_get_font_scale_factor:
- * @matrix: a #PangoMatrix, may be %NULL
- * 
- * Returns the scale factor of a matrix on the height of the font.
- * That is, the scale factor in the direction perpendicular to the
- * vector that the X coordinate is mapped to.
- *
- * Return value: the scale factor of @matrix on the height of the font,
- * or 1.0 if @matrix is %NULL.
- *
- * Since: 1.12
- **/
-double
-pango_matrix_get_font_scale_factor (const PangoMatrix *matrix)
-{
-/*
- * Based on cairo-matrix.c:_cairo_matrix_compute_scale_factors()
- *
- * Copyright 2005, Keith Packard
- */
-  double det;
-  
-  if (!matrix)
-    return 1.0;
-  
-  det = matrix->xx * matrix->yy - matrix->yx * matrix->xy;
-
-  if (det == 0)
-    {
-      return 0.0;
-    }
-  else
-    {
-      double x = matrix->xx;
-      double y = matrix->yx;
-      double major, minor;
-
-      major = sqrt (x*x + y*y);
-      
-      /*
-       * ignore mirroring
-       */
-      if (det < 0)
-	det = - det;
-      
-      if (major)
-	minor = det / major;
-      else 
-	minor = 0.0;
-
-      return minor;
-    }
-}
-
 static const char canon_map[256] = {
    0,   0,   0,   0,   0,   0,   0,   0,    0,   0,   0,   0,   0,   0,   0,   0, 
    0,   0,   0,   0,   0,   0,   0,   0,    0,   0,   0,   0,   0,   0,   0,   0, 
@@ -1259,6 +1204,7 @@ pango_language_copy (PangoLanguage *language)
 {
   return language; /* language tags are const */
 }
+
 static void
 pango_language_free (PangoLanguage *language)
 {
@@ -1339,7 +1285,7 @@ pango_language_from_string (const char *language)
  * or the range is a prefix of the tag, and the character after the
  * tag is '-'.
  *
- * Return value: %TRUE if a match was found.
+ * Returns: %TRUE if a match was found.
  **/
 gboolean
 pango_language_matches (PangoLanguage *language,
@@ -1455,28 +1401,84 @@ pango_language_get_sample_string (PangoLanguage *language)
   return result;
 }
 
-#ifdef HAVE_FRIBIDI
-
-gboolean
-pango_log2vis_get_embedding_levels (gunichar       *str,
-				    int             len,
-				    PangoDirection *pbase_dir,
-				    guint8         *embedding_level_list)
+guint8 *
+pango_log2vis_get_embedding_levels (const gchar    *str,
+				    int             bytelen,
+				    PangoDirection *pbase_dir)
 {
   FriBidiCharType fribidi_base_dir;
   gboolean result;
+  guint8 *embedding_levels_list;
 
-  fribidi_base_dir = (*pbase_dir == PANGO_DIRECTION_LTR) ? FRIBIDI_TYPE_L : FRIBIDI_TYPE_R;
+  switch (*pbase_dir)
+    {
+    case PANGO_DIRECTION_LTR:
+    case PANGO_DIRECTION_TTB_RTL:
+      fribidi_base_dir = FRIBIDI_TYPE_L;
+      break;
+    case PANGO_DIRECTION_RTL:
+    case PANGO_DIRECTION_TTB_LTR:
+      fribidi_base_dir = FRIBIDI_TYPE_R;
+      break;
+    case PANGO_DIRECTION_WEAK_RTL:
+      fribidi_base_dir = FRIBIDI_TYPE_WR;
+      break;
+    /*
+    case PANGO_DIRECTION_WEAK_LTR:
+    case PANGO_DIRECTION_NEUTRAL:
+    */
+    default:
+      fribidi_base_dir = FRIBIDI_TYPE_WL;
+      break;
+    }
+
+#ifdef FRIBIDI_HAVE_UTF8
+  {
+    if (bytelen < 0)
+      bytelen = strlen (str);
+    embedding_levels_list = fribidi_log2vis_get_embedding_levels_new_utf8 (str, bytelen, &fribidi_base_dir);
+  }
+#else
+  {
+    gunichar *text_ucs4;
+    int n_chars;
+    text_ucs4 = g_utf8_to_ucs4_fast (str, bytelen, &n_chars);
+    embedding_levels_list = g_new (guint8, n_chars);
+    fribidi_log2vis_get_embedding_levels ((FriBidiChar*)text_ucs4, n_chars,
+					  &fribidi_base_dir,
+					  (FriBidiLevel*)embedding_levels_list);
+    g_free (text_ucs4);
+  }
+#endif
 
-  result = fribidi_log2vis_get_embedding_levels (str, len, &fribidi_base_dir,
-						 embedding_level_list);
-  
   *pbase_dir = (fribidi_base_dir == FRIBIDI_TYPE_L) ?  PANGO_DIRECTION_LTR : PANGO_DIRECTION_RTL;
 
-  return result;
+  return embedding_levels_list;
 }
 
-#endif /* HAVE_FRIBIDI */
+/**
+ * pango_unichar_direction:
+ * @ch: a Unicode character
+ *
+ * Determines the direction of a character; either
+ * %PANGO_DIRECTION_LTR, %PANGO_DIRECTION_RTL, or
+ * %PANGO_DIRECTION_NEUTRAL.
+ *
+ * Return value: the direction of the character, as used in the
+ * Unicode bidirectional algorithm.
+ */
+PangoDirection
+pango_unichar_direction (gunichar ch)
+{
+  FriBidiCharType fribidi_ch_type = fribidi_get_type (ch);
+
+  if (!FRIBIDI_IS_LETTER (fribidi_ch_type))
+    return PANGO_DIRECTION_NEUTRAL;
+  else if (FRIBIDI_IS_RTL (fribidi_ch_type))
+    return PANGO_DIRECTION_RTL;
+  else
+    return PANGO_DIRECTION_LTR;
+}
 
 /**
  * pango_get_mirror_char:
@@ -1742,8 +1744,6 @@ pango_lookup_aliases (const char   *fontname,
  *
  * Return value: The direction corresponding to the first strong character.
  * If no such character is found, then %PANGO_DIRECTION_NEUTRAL is returned.
- *
- * Since: 1.4
  */
 PangoDirection
 pango_find_base_dir (const gchar *text,
@@ -1770,30 +1770,6 @@ pango_find_base_dir (const gchar *text,
   return dir;
 }
 
-/**
- * pango_unichar_direction:
- * @ch: a Unicode character
- *
- * Determines the direction of a character; either
- * %PANGO_DIRECTION_LTR, %PANGO_DIRECTION_RTL, or
- * %PANGO_DIRECTION_NEUTRAL.
- *
- * Return value: the direction of the character, as used in the
- * Unicode bidirectional algorithm.
- */
-PangoDirection
-pango_unichar_direction (gunichar ch)
-{
-  FriBidiCharType fribidi_ch_type = _pango_fribidi_get_type (ch);
-
-  if (!FRIBIDI_IS_LETTER (fribidi_ch_type))
-    return PANGO_DIRECTION_NEUTRAL;
-  else if (FRIBIDI_IS_RTL (fribidi_ch_type))
-    return PANGO_DIRECTION_RTL;
-  else
-    return PANGO_DIRECTION_LTR;
-}
-
 /**
  * pango_is_zero_width:
  * @ch: a Unicode character
@@ -1802,8 +1778,6 @@ pango_unichar_direction (gunichar ch)
  * normally rendered on the screen.
  *
  * Return value: %TRUE if @ch is a zero-width character, %FALSE otherwise
- *
- * Since: 1.10
  */
 gboolean
 pango_is_zero_width (gunichar ch)
