@@ -47,6 +47,7 @@ pango_cairo_font_map_get_type (void)
 	NULL,		/* class_data */
 	0,
 	0,
+	NULL,
 	NULL
       };
 
@@ -74,8 +75,8 @@ pango_cairo_font_map_get_type (void)
  * You generally should only use the #PangoFontMap and
  * #PangoCairoFontMap interfaces on the returned object.
  * 
- * Return value: the newly allocated #PangoFontMap, which should
- *               be freed with g_object_unref().
+ * Return value: the newly created fontmap object. Free
+ * with g_object_unref().
  *
  * Since: 1.10
  **/
@@ -220,8 +221,14 @@ static PangoCairoContextInfo *
 get_context_info (PangoContext *context,
 		  gboolean      create)
 {
-  PangoCairoContextInfo *info = g_object_get_data (G_OBJECT (context),
-						   "pango-cairo-context-info");
+  static GQuark context_info_quark;
+  PangoCairoContextInfo *info;
+
+  if (!context_info_quark)
+    context_info_quark = g_quark_from_static_string ("pango-cairo-context-info");
+
+  info = g_object_get_qdata (G_OBJECT (context), context_info_quark);
+
   if (!info && create)
     {
       info = g_new (PangoCairoContextInfo, 1);
@@ -230,8 +237,8 @@ get_context_info (PangoContext *context,
       info->surface_options = NULL;
       info->merged_options = NULL;
 
-      g_object_set_data_full (G_OBJECT (context), "pango-cairo-context-info", 
-			      info, (GDestroyNotify)free_context_info);
+      g_object_set_qdata_full (G_OBJECT (context), context_info_quark, 
+			       info, (GDestroyNotify)free_context_info);
     }
 
   return info;
@@ -254,7 +261,7 @@ void
 pango_cairo_update_context (cairo_t      *cr,
 			    PangoContext *context)
 {
-  PangoCairoContextInfo *info = get_context_info (context, TRUE);
+  PangoCairoContextInfo *info;
   cairo_matrix_t cairo_matrix;
   cairo_surface_t *target;
   PangoMatrix pango_matrix;
@@ -262,6 +269,8 @@ pango_cairo_update_context (cairo_t      *cr,
   g_return_if_fail (cr != NULL);
   g_return_if_fail (PANGO_IS_CONTEXT (context));
 
+  info = get_context_info (context, TRUE);
+
   cairo_get_matrix (cr, &cairo_matrix);
   pango_matrix.xx = cairo_matrix.xx;
   pango_matrix.yx = cairo_matrix.yx;
@@ -372,7 +381,7 @@ pango_cairo_context_set_font_options (PangoContext               *context,
 }
   
 /**
- * pango_cairo_context_get_font_options:
+ * pango_cairo_get_font_options:
  * @context: a #PangoContext, from pango_cairo_font_map_create_context()
  * 
  * Retrieves any font rendering options previously set with
