@@ -483,9 +483,6 @@ pango_fc_font_map_finalize (GObject *object)
   g_queue_free (priv->fontset_cache);
   g_hash_table_destroy (priv->coverage_hash);
 
-  if (priv->fontset_hash)
-    g_hash_table_destroy (priv->fontset_hash);
-
   if (priv->font_hash)
     g_hash_table_destroy (priv->font_hash);
 
@@ -1011,6 +1008,44 @@ get_unscaled_size (PangoFcFontMap             *fcfontmap,
     }
 }
 
+/*
+ * Based on cairo-matrix.c:_cairo_matrix_compute_scale_factors()
+ *
+ * Copyright 2005, Keith Packard
+ */
+static int
+get_scaled_size (FontsetHashKey *key)
+{
+  PangoMatrix *matrix = &key->matrix;
+  double det = matrix->xx * matrix->yy - matrix->yx * matrix->xy;
+
+  if (det == 0)
+    {
+      return 0.0;
+    }
+  else
+    {
+      double x = matrix->xx;
+      double y = matrix->yx;
+      double major, minor;
+
+      major = sqrt (x*x + y*y);
+      
+      /*
+       * ignore mirroring
+       */
+      if (det < 0)
+	det = - det;
+      
+      if (major)
+	minor = det / major;
+      else 
+	minor = 0.0;
+
+      return minor * (key->size / 1024.);
+    }
+}
+
 static PangoFcPatternSet *
 pango_fc_font_map_get_patterns (PangoFontMap               *fontmap,
 				PangoContext               *context,
@@ -1045,10 +1080,8 @@ pango_fc_font_map_get_patterns (PangoFontMap               *fontmap,
 
   if (patterns == NULL)
     {
-      double scale_factor = pango_matrix_get_font_scale_factor (&key.matrix);
-      double scaled_size = key.size * scale_factor / PANGO_SCALE;
       pattern = pango_fc_make_pattern (desc, language,
-				       scaled_size,
+				       get_scaled_size (&key),
 				       pango_fc_font_map_get_resolution (fcfontmap, context));
       
       pango_fc_default_substitute (fcfontmap, context, pattern);
@@ -1804,6 +1837,7 @@ pango_fc_face_get_type (void)
         sizeof (PangoFcFace),
         0,              /* n_preallocs */
         (GInstanceInitFunc) NULL,
+	NULL            /* value_table */
       };
       
       object_type = g_type_register_static (PANGO_TYPE_FONT_FACE,
@@ -1931,6 +1965,7 @@ pango_fc_family_get_type (void)
         sizeof (PangoFcFamily),
         0,              /* n_preallocs */
         (GInstanceInitFunc) pango_fc_family_init,
+	NULL            /* value_table */
       };
       
       object_type = g_type_register_static (PANGO_TYPE_FONT_FAMILY,
