@@ -36,12 +36,6 @@
  */
 
 
-/* Keep these in sync with the same macros in break.c */
-
-#define LEADING_JAMO(wc)  ((wc) >= 0x1100 && (wc) <= 0x115F)
-#define VOWEL_JAMO(wc)    ((wc) >= 0x1160 && (wc) <= 0x11A2)
-#define TRAILING_JAMO(wc) ((wc) >= 0x11A8 && (wc) <= 0x11F9)
-#define JAMO(wc)          ((wc) >= 0x1100 && (wc) <= 0x11FF)
 /* "virama script" is just an optimization; it includes a bunch of
  * scripts without viramas in them
  */
@@ -72,9 +66,8 @@ static gunichar current_wc = 0;
 static const char *line_start = NULL;
 static const char *line_end = NULL;
 
-static void
-fail (const char *format,
-      ...)
+static void fail (const char *format, ...) G_GNUC_PRINTF (1, 2) G_GNUC_NORETURN;
+static void fail (const char *format, ...)
 {
   char *str;
   char *line_text;
@@ -281,6 +274,8 @@ check_grapheme_invariants (const char   *text,
 
 }
 
+static void print_sentences (const char   *text,
+                             PangoLogAttr *attrs);
 static void
 print_sentences (const char   *text,
                  PangoLogAttr *attrs)
@@ -339,12 +334,11 @@ check_invariants (const char *text)
 }
 
 int
-main (int    argc,
-      char **argv)
+main (int argc, char *argv[]) 
 {
   gchar *text;
-  gchar *srcdir;
-  gchar *filename;
+  const gchar *srcdir;
+  const gchar *filename;
 
   srcdir = getenv ("srcdir");
   if (!srcdir)
